% This is to test that one run of general_least_squares_error produces the
% correct outputs.

classdef GeneralLSETest < matlab.unittest.TestCase
    properties
        % [L, strain_data] = general_least_squares_error(theta)
        theta
        L
        strain_data

        % global variables in general_least_squares_error.m
        counter
        fix_b
        fix_P
        production_function

        % global variables in general_network_dynamics_sigmoid
        degrate
        num_genes
    end

    methods (TestClassSetup)
        function addGRNmapPath (testCase) %#ok<MANU>
            addpath([pwd '\..\..\..\matlab'])
        end

        function setupConstantVariables (testCase)
            testCase.counter = 0;
            testCase.fix_b = 0;
            testCase.fix_P = 0;
            testCase.production_function = 'Sigmoid';
        end

        function runGLSE (testCase)
            global counter fix_b fix_P production_function

            counter               = testCase.counter;
            fix_b                 = testCase.fix_b;
            fix_P                 = testCase.fix_P;
            production_function   = testCase.production_function;
        end
    end

    methods (TestClassTeardown)
        function teardown (testCase) %#ok<MANU>
            clearvars -global
        end
    end

    methods (Test)
        function testLIsCorrectFor3Gene3EdgesControlCase (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0
                              0 1 0
                              0 0 1 ];
            alpha = 0;
            b = [0; 0; 0];
            degrate = [0.5; 0.8; 1];
            expression_timepoints = [5 10 20];
            num_genes = 3;
            is_forced = [1; 2; 3];
            log2FC = struct (...
                'Strain', {{'wt'}},...
                'deletion', {0}, ...
                't', struct ('indx', {[1 2 3 4]; [5 6 7 8]; [9 10 11 12 13]}, 't', {5; 10; 20}), ...
                'raw', {[5  5  5  5 10 10 10 10 20 20 20 20 20
                          0  0  0  0  0  0  0  0  0  0  0  0  0
                          0  0  0  0  0  0  0  0  0  0  0  0  0
                          0  0  0  0  0  0  0  0  0  0  0  0  0
                         ]},...
                'compressed', {{
                    5, 10, 20;
                    [0  0  0  0; 1 2 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                    [0  0  0  0; 1 2 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                    [0  0  0  0; 1 2 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                }}...
            );
            strain_length = 1;
            testCase.theta = [0; 0; 0; 0; 0; 0; 1; 1.6; 2];
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyEqual(testCase.L, 0);
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainControlCase (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ]},...
                 'compressed', {{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],   [0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],   [0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],   [0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];
                    [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]
                  }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - 0) < 1E-08); % round because relative error = 1E-12
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsControlCase (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                 'compressed', {{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],   [0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],   [0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],   [0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];
                    [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]
                 };{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [0  0  0; 1 2 3],  [0  0  0; 1 2 3],  [0  0  0; 1 2 3],  [0  0  0; 1 2 3];
                    [-0.13932150464649  -0.13932150464649  -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                 }} ...
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - 0) < 1E-14); % abs(testCase.L - 0) = 2.1e-15
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAdd1ToData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           1.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]},...
                  'compressed', {{
                      0.4,  0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                      [ 1.269293821322375   0.269293821322375   0.269293821322375; 1 2 3], [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3], [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557;  1 2 3], [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 1/48;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 9.8924e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAdd1ToData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'};{'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           1.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                  'compressed', {{
                     0.4,  0.8,  1.2,  1.6;...
                     [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                     [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                     [ 1.269293821322375   0.269293821322375   0.269293821322375; 1 2 3], [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],  [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                     [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3], [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3], [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  };{
                     0.4,  0.8,  1.2, 1.6
                     [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3], [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                     [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3], [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3]
                     [ 0                   0                   0; 1 2 3],                 [0                    0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                  0                  0; 1 2 3]
                     [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3], [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106 -0.402590113617106 -0.402590113617106; 1 2 3]
                  }} ...          
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 1/96;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 9.8925e-08
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAdd2ToData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 2.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]},...
                 'compressed', {{0.4,  0.8,  1.2, 1.6;...
                          [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                          [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                          [ 0.269293821322375   2.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],  [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                          [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557;  1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                 }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            expectedL = 4/48;
            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 4.9462e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAdd2ToData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 2.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                 'compressed', {{
                      0.4,  0.8,  1.2, 1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   2.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],  [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557;  1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                 };{
                      0.4, 0.8,  1.2,  1.6
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3]
                      [ 0                   0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3]
                      [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                 }}...
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            expectedL = 4/96;
            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 4.9462e-08
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAdd1ToTwoDataPoints (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           1.269293821322375	 1.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]}, ...
                 'compressed', {{
                      0.4,  0.8,  1.2, 1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052],  [-0.706666467343382  -0.706666467343382  -0.706666467343382],  [-0.987239299670092  -0.987239299670092  -0.987239299670092], [-1.21744326547615  -1.21744326547615  -1.21744326547615];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836],  [-0.408017743026565  -0.408017743026565  -0.408017743026565],  [-0.546412653239117  -0.546412653239117  -0.546412653239117], [-0.649843860182989  -0.649843860182989  -0.649843860182989];...
                      [ 1.269293821322375   1.269293821322375   0.269293821322375],  [ 0.415935372482249   0.415935372482249   0.415935372482249],  [ 0.497511633589691   0.497511633589691   0.497511633589691], [ 0.541599938412416   0.541599938412416   0.541599938412416];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703],  [-0.579771801563864  -0.579771801563864  -0.579771801563864],  [-0.85583465653557   -0.85583465653557   -0.85583465653557],  [-1.10972623205696  -1.10972623205696  -1.10972623205696]...
                 }}...   
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 2/48;
            testCase.verifyTrue(abs((testCase.L - expectedL)/expectedL) < 1E-07); % round because relative error = 9.8924e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAdd1ToSameStrains (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           1.269293821322375	 1.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                  'compressed', {{
                      0.4,  0.8,  1.2, 1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                      [ 1.269293821322375   1.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  };{
                      0.4, 0.8,  1.2,  1.6
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3]
                      [ 0                   0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3]
                      [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                  }}...
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 2/96;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 9.8924e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAdd1ToDifferentStrains (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0.269293821322375	 1.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                           0.86067849535351	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                  'compressed', {{
                      0.4,  0.8,  1.2, 1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   1.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  };{
                      0.4, 0.8,  1.2,  1.6
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3]
                      [ 0                   0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3]
                      [0.86067849535351    -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                  }}...    
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 2/96;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 3.3011e-08
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAlphaIs1(testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 1;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]}, ...
                 'compressed', {{
                      0.4, 0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416  0.541599938412416  0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  }}...    
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            manualPenaltyOutCalculation = 0.7008928571 * alpha;
            expectedL = manualPenaltyOutCalculation;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because we keep getting L = 9.8e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAlphaIs1(testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 1;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]}, ...
                  'compressed', {{
                      0.4, 0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416  0.541599938412416  0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                   };{
                      0.4, 0.8,  1.2,  1.6
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3]
                      [ 0                   0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3]
                      [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                  }}...
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            manualPenaltyOutCalculation = 0.7008928571 * alpha;
            expectedL = manualPenaltyOutCalculation;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-10); % round because relative error = 6.1149e-11
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAlphaIs2(testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];

            alpha = 2;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]}, ...
                 'compressed', {{
                      0.4, 0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416  0.541599938412416  0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                 }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            manualPenaltyOutCalculation = 0.7008928571 * alpha;
            expectedL = manualPenaltyOutCalculation;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-10); % round because relative error = 6.1148e-11
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAlphaIs2(testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];

            alpha = 2;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]}, ...
                 'compressed', {{
                      0.4, 0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3];...
                      [ 0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],  [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416  0.541599938412416  0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                 };{
                      0.4, 0.8,  1.2,  1.6
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3]
                      [ 0                   0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3],                  [ 0                   0                   0; 1 2 3]
                      [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                 }}...    
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            manualPenaltyOutCalculation = 0.7008928571 * alpha;
            expectedL = manualPenaltyOutCalculation;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-10); % round because relative error = 6.1148e-11
        end
        
        function testLIsCorrectFor3Gene3EdgesMissingData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0
                              0 1 0
                              0 0 1 ];
            alpha = 0;
            b = [0; 0; 0];
            degrate = [0.5; 0.8; 1];
            expression_timepoints = [5 10 20];
            num_genes = 3;
            is_forced = [1; 2; 3];
            log2FC = struct (...
                'Strain', {{'wt'}},...
                'deletion', {0}, ...
                't', struct ('indx', {[1 2 3 4]; [5 6 7 8]; [9 10 11 12 13]}, 't', {5; 10; 20}), ...
                'raw', {[5  5  5  5 10 10 10 10 20 20 20 20 20
                          0  0  0  0  0  0  0  0  0  0  0  0  0
                          0 NaN 0  0  0  0  0  0  0  0  0  0  0
                          0  0  0  0  0  0  0  0  0  0  0  0  0
                         ]},...
                'compressed', {{
                    5, 10, 20;
                    [0  0  0  0; 1 2 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                    [0  0  0; 1 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                    [0  0  0  0; 1 2 3 4], [0  0  0  0; 1 2 3 4], [0  0  0  0  0; 1 2 3 4 5];
                }}...
            );
            strain_length = 1;
            testCase.theta = [0; 0; 0; 0; 0; 0; 1; 1.6; 2];
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyEqual(testCase.L, 0);
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainMissingData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           NaN               	 NaN                 0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ]},...
                 'compressed', {{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [ 0.269293821322375; 3],   [0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],   [0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],   [0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];
                    [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]
                  }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - 0) < 1E-08); % round because relative error = 1E-12
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsMissingData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}; {'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	NaN              	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0.269293821322375	 0.269293821322375   0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	NaN              	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     0                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                 'compressed', {{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836; 1 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [0.269293821322375   0.269293821322375   0.269293821322375; 1 2 3],   [0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],   [0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],   [0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];
                    [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3],  [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3],  [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]
                 };{
                    0.4,  0.8,  1.2,  1.6;
                    [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3],  [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];
                    [-0.227718654239836  -0.227718654239836; 1 3],  [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];
                    [0  0  0; 1 2 3],  [0  0  0; 1 2 3],  [0  0  0; 1 2 3],  [0  0  0; 1 2 3];
                    [-0.13932150464649  -0.13932150464649  -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3],  [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106  -0.402590113617106  -0.402590113617106; 1 2 3]
                 }} ...
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            testCase.verifyTrue(abs(testCase.L - 0) < 1E-14); % abs(testCase.L - 0) = 2.1e-15
        end

        function testLIsCorrectFor4Gene6EdgesOneStrainAdd1ToMissingData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'}},...
                 'deletion', {0}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           1.269293821322375	 NaN                 0.269293821322375	 0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ]},...
                  'compressed', {{
                      0.4,  0.8,  1.2,  1.6;...
                      [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3], [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                      [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3], [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                      [ 1.269293821322375   0.269293821322375; 1 3], [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3], [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                      [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3], [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557   -0.85583465653557   -0.85583465653557;  1 2 3], [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  }}...
            );
            strain_length = 1;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 1/47;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 9.8924e-08
        end

        function testLIsCorrectFor4Gene6EdgesTwoStrainsAdd1ToMissingData (testCase)
            global adjacency_mat alpha b degrate expression_timepoints ...
                num_genes is_forced log2FC strain_length

            adjacency_mat = [ 1 0 0 0
                              0 1 0 0
                              0 0 1 1
                              0 0 1 1 ];
            alpha = 0;
            b = [0; 0; 0; 0];
            degrate = [1; 1; 1; 1];
            expression_timepoints = [0.4 0.8 1.2 1.6];
            num_genes = 4;
            is_forced = [1; 2; 3; 4];
            log2FC = struct (...
                 'Strain', {{'wt'};{'dcin5'}},...
                 'deletion', {0; 3}, ...
                 't', struct ('indx', {[1 2 3]; [4 5 6]; [7 8 9]; [10 11 12]}, 't', {0.4; 0.8; 1.2; 1.6}), ...
                 'raw', {[0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6;...
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615;...
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989;...
                           1.269293821322375	 0.269293821322375   NaN	             0.415935372482249	 0.415935372482249	 0.415935372482249	 0.497511633589691	 0.497511633589691	 0.497511633589691	 0.541599938412416	 0.541599938412416	 0.541599938412416;...
                          -0.290936155220703	-0.290936155220703	-0.290936155220703	-0.579771801563864	-0.579771801563864	-0.579771801563864	-0.85583465653557	-0.85583465653557	-0.85583465653557	-1.10972623205696	-1.10972623205696	-1.10972623205696...
                          ];
                          [0.4	0.4	0.4	0.8	0.8	0.8	1.2	1.2	1.2	1.6	1.6	1.6
                          -0.376334306292052	-0.376334306292052	-0.376334306292052	-0.706666467343382	-0.706666467343382	-0.706666467343382	-0.987239299670092	-0.987239299670092	-0.987239299670092	-1.21744326547615	-1.21744326547615	-1.21744326547615
                          -0.227718654239836	-0.227718654239836	-0.227718654239836	-0.408017743026565	-0.408017743026565	-0.408017743026565	-0.546412653239117	-0.546412653239117	-0.546412653239117	-0.649843860182989	-0.649843860182989	-0.649843860182989
                           0                     NaN                   0                   0                   0                   0                   0                   0                   0                   0                   0                   0
                          -0.13932150464649	    -0.13932150464649	-0.13932150464649	-0.249851730445413	-0.249851730445413	-0.249851730445413	-0.336140688374095	-0.336140688374095	-0.336140688374095	-0.402590113617106	-0.402590113617106	-0.402590113617106
                          ]},...
                  'compressed', {{
                     0.4,  0.8,  1.2,  1.6;...
                     [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3],  [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3];...
                     [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3],  [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989  -0.649843860182989  -0.649843860182989; 1 2 3];...
                     [ 1.269293821322375   0.269293821322375; 1 2], [ 0.415935372482249   0.415935372482249   0.415935372482249; 1 2 3],  [ 0.497511633589691   0.497511633589691   0.497511633589691; 1 2 3],  [ 0.541599938412416   0.541599938412416   0.541599938412416; 1 2 3];...
                     [-0.290936155220703  -0.290936155220703  -0.290936155220703; 1 2 3], [-0.579771801563864  -0.579771801563864  -0.579771801563864; 1 2 3],  [-0.85583465653557  -0.85583465653557  -0.85583465653557; 1 2 3], [-1.10972623205696  -1.10972623205696  -1.10972623205696; 1 2 3]...
                  };{
                     0.4,  0.8,  1.2, 1.6
                     [-0.376334306292052  -0.376334306292052  -0.376334306292052; 1 2 3], [-0.706666467343382  -0.706666467343382  -0.706666467343382; 1 2 3], [-0.987239299670092  -0.987239299670092  -0.987239299670092; 1 2 3],  [-1.21744326547615  -1.21744326547615  -1.21744326547615; 1 2 3]
                     [-0.227718654239836  -0.227718654239836  -0.227718654239836; 1 2 3], [-0.408017743026565  -0.408017743026565  -0.408017743026565; 1 2 3], [-0.546412653239117  -0.546412653239117  -0.546412653239117; 1 2 3],  [-0.649843860182989 -0.649843860182989 -0.649843860182989; 1 2 3]
                     [ 0                   0; 1 3],                 [0                    0                   0; 1 2 3],                 [ 0                   0                   0; 1 2 3],                  [ 0                  0                  0; 1 2 3]
                     [-0.13932150464649   -0.13932150464649   -0.13932150464649; 1 2 3],  [-0.249851730445413  -0.249851730445413  -0.249851730445413; 1 2 3], [-0.336140688374095  -0.336140688374095  -0.336140688374095; 1 2 3],  [-0.402590113617106 -0.402590113617106 -0.402590113617106; 1 2 3]
                  }} ...          
            );
            strain_length = 2;
            testCase.theta = [0.5; 0.25; 0.5; 1.0; -1.0; 1.0; % Network weights
                              0.0; 0.0;  0.0; 0.0;            % Threshold
                              0.5; 1.0;  2.0; 1.0];           % Production rates
            [testCase.L, testCase.strain_data] = general_least_squares_error(testCase.theta);

            % we need to test for strain_data later
            expectedL = 1/94;
            testCase.verifyTrue(abs(testCase.L - expectedL)/expectedL < 1E-07); % round because relative error = 9.8925e-08
        end
    end
end
